# encoding: utf-8

import conf
from clirch.core.pipeline import Pipelines

class Clirch(object):
    """
    CLIRch main class.

    This is the object that should be instanciated for standard use of the full
    framework. Configuration can be loaded using the Config class from
    clirch.conf and passing an instance to the object. e.g:

    >>> from clirch import Clirch
    >>> from clirch.conf import Config
    >>> from clirch.languages import norwegian, spanish
    >>> config_instance = Config({'languages': [norwegian.language_class,
    ...         spanish.language_class]})
    >>> bh_instance = Clirch(config_instance)
    """

    def __init__(self, config_instance=None):
        self.from_languages = dict()
        if config_instance is None:
            self.config_instance = conf.Config()
        else:
            self.config_instance = config_instance

        for language in self.config_instance.languages.values():
            # TODO: Build the pipeline using only needed parameters?
            pipelines = Pipelines(language, self.config_instance)
            self.from_languages[language.code] = pipelines
            setattr(self, 'from_%s' % language.name.lower(), pipelines)
