# encoding: utf-8

"""
The language module of CLIRch.

Contains all language specific code.

If you are not developing directly on CLIRch, you can still register
custom languages:

    >>> from clirch.languages import norwegian
    >>> from my_language_module import swahili
    >>> config_instance = Config({
    ...     'languages': [norwegian, swahili],
    ...     'pipeline_overrides': <something>
    ...     })
    >>> bh = Clirch(config_instance)
"""

from nltk import lazyimport

from main import Language
from clirch import utils, lookup, pruning

builtin_languages = ['spanish', 'norwegian', 'english']

registry = lazyimport.LazyModule('registry', locals())

english = Language(name='English', code='en',
        in_pipeline=[
            utils.force_unicode,
            utils.word_tokenize,
            utils.get_stopword_filtrer('English'),
            utils.get_stemmer('English'),
            lookup.wordnet_en.wordnet_synonyms,
        ],
        translation=[
            lookup.pickled.PickledDict
        ],
        out_pipeline=[
            pruning.eliminate_duplicates,
            pruning.reverse.ReversePruner(lookup.pickled.PickledDict),
        ],
    )

norwegian = Language(name='Norwegian', code='no',
        in_pipeline=[
            utils.force_unicode,
            utils.mtag,
            utils.get_stopword_filtrer('Norwegian'),
        ],
        translation=[
            lookup.pickled.PickledDict
        ],
        out_pipeline=[
            pruning.reduce_options_by_group(0, 1),
        ]
    )

spanish = Language(name='Spanish', code='es',
        in_pipeline=[
            utils.force_unicode,
            utils.freeling('es'),
            utils.get_stopword_filtrer('Spanish'),
        ],
        translation=[
            lookup.pickled.PickledDict
        ],
        out_pipeline={'no': [
                pruning.reduce_options_by_group(0, 1),
            ], 'default': [
                pruning.eliminate_duplicates,
                pruning.reverse.ReversePruner(lookup.pickled.PickledDict, level=0),
            ]
        }
    )
