# encoding: utf-8

import copy

class Language(object):
    """
    All languages inherit from here.
    Not to be instanciated directly.
    """

    def __init__(self, name, code,
            in_pipeline, translation, out_pipeline, config=None):
        self.name = name
        self.code = code
        self.in_pipeline = in_pipeline
        self.translation = translation
        self.out_pipeline = out_pipeline

        self.pipes = dict()
        self.config = None
        if config is not None:
            self.update_config(config)

    def __unicode__(self):
        return self.name

    def __repr__(self):
        return '<clirch.language.Language: "%s">' % self.name

    def set_pipeline(self, language, pipeline):
        self.default_pipes[language] = pipeline

    def update_config(self, config=None):
        if config is not None:
            self.config = config
        self._construct_default_pipelines()

    def _construct_default_pipelines(self):
        for code, language in self.config.languages.iteritems():
            self.pipes[language] = list()
            for pipe_opts in [self.in_pipeline, self.translation, self.out_pipeline]:
                if isinstance(pipe_opts, dict):
                    self.pipes[language] += pipe_opts.get(code, pipe_opts.get('default'))
                else:
                    self.pipes[language] += pipe_opts
            self.pipes.update(self.config.pipeline_overrides.get(
                self, dict()))
        for language, pipeline in self.pipes.iteritems():
            new_pipeline = []
            for component in pipeline:
                if isinstance(component, type):
                    component = component(self.code, language.code)
                elif hasattr(component, 'set_languages'):
                    component = copy.copy(component)
                    component.set_languages(self.code, language.code)
                new_pipeline.append(component)
            self.pipes[language] = new_pipeline
