# encoding: utf-8

from clirch import language

class LanguageRegistry(dict):
    """
    Class for registering languages under a config object of CLIRch.
    """
    def __init__(self, languages=None, config=None):
        self.config = config

        if languages is None:
            for language_name in language.builtin_languages:
                lang = getattr(language, language_name)
                self[lang.code] = lang
        else:
            if not isinstance(languages, dict):
                languages = [(lang.code, lang) for
                        lang in languages]
            self.update(languages)

        if config is not None:
            config.languages = self
            self.update_config()

    def add_language(self, lang):
        if lang.code in self:
            raise TypeError('LanguageRegistry instances do not support '
                    'overwriting existing languages')

        lang.update_config(self.config)

        self[lang.code] = lang

    def update_config(self, config=None):
        if config is not None:
            self.config = config

        for lang in self.values():
            lang.update_config(self.config)
