#!/usr/bin/env python

from collections import defaultdict

try:
    from xml.etree import cElementTree as ElementTree
except ImportError:
    from xml.etree import ElementTree

try:
    import cPickle as pickle
except ImportError:
    import pickle

ARTICLE_TAG = 'pe_article'
META_TAG = 'm'
META_ATTRIB = 'mn'
META_TITLE_VALUE = 'title'
META_ID_VALUE = 'id'
EQUIVALENT_TAG = 'ekvivalent'
TRANSLATION_TAG = 'uttrykksoversettelse'
EXAMPLE_TAG = 'eksempeloversettelse'
LINK_FROM = 'pe_source'
LINK_TO = 'pe_target'

def extract_text(node):
    text = node.text
    for child in node:
        if child.tail:
            text += child.tail
    return text

def dict_from_xml(filename):
    tree = ElementTree.ElementTree(file=filename)
    dictionary = defaultdict(list)
    id_to_titles = dict()
    article_links = dict()
    for article in tree.iterfind(ARTICLE_TAG):
        titles = ''
        word_id = None
        raw_translations = list()
        link_from = None
        lacks_equivalent = True
        lacks_translation = True
        for node in article.getiterator():
            if node.tag == META_TAG:
                if node.attrib[META_ATTRIB] == META_TITLE_VALUE:
                    titles = node.text
                if node.attrib[META_ATTRIB] == META_ID_VALUE:
                    word_id = int(node.text.strip())
            elif node.tag == EQUIVALENT_TAG:
                lacks_equivalent = False
                raw_translations.append(extract_text(node))
            elif node.tag == TRANSLATION_TAG and lacks_equivalent:
                lacks_translation = False
                raw_translations.append(extract_text(node))
            elif node.tag == EXAMPLE_TAG and lacks_equivalent and lacks_translation:
                raw_translations.append(extract_text(node))
            elif node.tag == LINK_FROM:
                link_from = int(node.text.strip())
            elif node.tag == LINK_TO:
                article_links[link_from] = int(node.text.strip())

        translations = list()
        for raw_translation in raw_translations:
            if raw_translation and raw_translation.strip():
                translation_parts = list()
                if '(' in raw_translation:
                    part1, part2 = raw_translation.split('(', 1)
                    part2, part3 = part2.split(')', 1)
                    translation_parts.append(' '.join((part1 + part2 + part3).split()))
                    translation_parts.append(' '.join([part1, part3]))
                else:
                    translation_parts.append(raw_translation)
                for translation in translation_parts:
                    if '/' in translation:
                        part1, part2 = translation.split('/', 1)
                        sword1, sword2 = part1.split()[-1], part2.split()[0]
                        translations.append(u' '.join(part1.split()[:-1] + [sword1] + part2.split()[1:]))
                        translations.append(u' '.join(part1.split()[:-1] + [sword2] + part2.split()[1:]))
                    else:
                        translations.append(u' '.join(translation.split()))

        for title in (part.strip() for part in unicode(titles).lower().split(',')):
            if title:
                dictionary[title] += translations
                id_to_titles[word_id] = title

    for link_from, link_to in article_links.iteritems():
        try:
            dictionary[id_to_titles[link_from]] += dictionary[id_to_titles[link_to]]
            #dictionary[id_to_titles[link_to]] = dictionary[id_to_titles[link_from]]
        except KeyError:
            pass

    return dict(dictionary)

def dict_to_pickle(dictionary, filename):
    with open(filename, 'w') as fh:
        pickle.dump(dictionary, fh)

if __name__ == '__main__':
    import sys
    if len(sys.argv) > 1:
        filenames = sys.argv[1:]
    else:
        filenames = ['norsk-spansk.xml']

    for filename in filenames:
        dictionary = dict_from_xml(filename)
        for _, entry in zip(xrange(30), dictionary.iteritems()):
            key, value = entry
            print key + ':\t' + u' ; '.join(value)
        dict_to_pickle(dictionary, filename + '.pickle')
