# encoding: utf-8

from __future__ import with_statement

from collections import defaultdict
import os

try:
    import cPickle as pickle
except ImportError:
    import pickle

from clirch.core import logger
from clirch.utils.word import TransformedWord
import dictionary

# TODO: This is config, doesn't belong here!
DICTIONARIES = {
    'no': {'es': os.path.join(os.path.dirname(__file__), 'norsk-spansk.xml.pickle')},
    'es': {'no': os.path.join(os.path.dirname(__file__), 'spansk-norsk.xml.pickle')},
}

_cache = defaultdict(dict)

def _get_dict(from_lang, to_lang):
    global _cache
    if from_lang in _cache and to_lang in _cache[from_lang]:
        return _cache[from_lang][to_lang]
    try:
        pickle_file = DICTIONARIES[from_lang][to_lang]
    except KeyError:
        pickle_file = os.path.join(os.path.dirname(__file__), '%s_%s.pickle' % (from_lang, to_lang))
    if os.path.exists(pickle_file):
        logger.debug('Loading PickledDict cache from ' + pickle_file)
        with open(pickle_file, 'r') as fh:
            _cache[from_lang][to_lang] = pickle.load(fh)
            return _cache[from_lang][to_lang]
    else:
        logger.debug('File not found: ' + pickle_file)

class PickledDict(dictionary.Dictionary):
    """
    Load a dictionary from a python dict in a pickle file
    """
    def __init__(self, from_lang, to_lang, passthrough=True, fallback=True):
        self.from_lang = from_lang
        self.to_lang = to_lang
        self.passthrough = passthrough
        self.fallback = fallback
        self._cache = _get_dict(from_lang, to_lang)
        if self._cache is None:
            self._cache = dict()
            self.passthrough = True
            self.fallback = False

    def __call__(self, words):
        for word in words:

            # TODO: Oh my! We should make a generalized Backoff() util for
            # maximum efficiency here! C'est génial! Also, we could/should do
            # ABC->Backoff(['ABC'],['AB','BC'],['A','B','C']) – 'ain søkke!
            # ¡Oye!
            # [' '.join(subsent) for n in xrange(len(words), 0, -1) for subsent in nltk.ngrams(words, n)]
            # Well, no, you have to be smarter – never reuse words!
            if self.passthrough:
                word_options = getattr(word, 'origins', []) + [word]
            else:
                word_options = [word]

            result = []
            used = []
            for option in word_options:
                if option in used:
                    continue
                used.append(option)
                if option in self._cache:
                    result += self._cache[option]
                    if not self.fallback:
                        break

            if not result and self.passthrough:
                result = [word]

            logger.debug('PickledDict: %s -> %s' % (repr(word), repr(result)))
            for translation in result:
                if translation is word:
                    # If using passthrough, give a moderate weightdown
                    yield TransformedWord(translation.strip('-'), word, .96)
                else:
                    yield TransformedWord(translation.strip('-'), word)
