# encoding: utf-8
"""
Methods used for looking up in WordNet (English)
"""

from nltk.corpus import wordnet

from clirch.utils.word import TransformedWord

def wordnet_synonyms_one_word(word):
    "Looks up synset lemmas for a single word in nltk.corpus.wordnet"
    for ss in wordnet.synsets(word):
        for lemma in ss.lemma_names:
            yield lemma

def wordnet_synonyms(words):
    "wordnet_synonyms_one_word for each word"
    for word in words:
        synonyms = list(wordnet_synonyms_one_word(word))
        if not synonyms and getattr(word, 'origins', False):
            for synonym in word.origins:
                synonyms = list(wordnet_synonyms_one_word(synonym))
                if synonyms:
                    break
        if not synonyms:
            synonyms = [word]
        for synonym in synonyms:
            yield TransformedWord(synonym.replace('_', ' ').lower(), word)
