# encoding: utf-8
"""
Methods used for looking up in WordNet (Spanish)
"""

from __future__ import with_statement

from collections import defaultdict

from clirch.utils.word import TransformedWord

DEFAULT_SENSEFILE='/usr/share/FreeLing/es/senses30.src'

_sid_cache = defaultdict(dict)
_word_cache = defaultdict(dict)

class SpanishWordnetSynonyms(object):

    def __init__(self, sensefile=None):
        self.sensefile = sensefile or DEFAULT_SENSEFILE

        if _word_cache:
            return

        word_cache, sid_cache = _word_cache, _sid_cache

        with open(self.sensefile) as senses:
            for line in senses:
                key, value = line.split(' ', 1)
                key = key[2:-2]
                if line.startswith('S'):
                    sid = int(key)
                    for word in unicode(value, 'iso-8859-1').split():
                        word = word.replace('_', ' ')
                        sid_cache[sid][word] = word_cache[word]
                elif line.startswith('W'):
                    word = unicode(key.replace('_', ' '), 'iso-8859-1')
                    for sid in value.split():
                        sid = int(sid)
                        word_cache[word][sid] = sid_cache[sid]

    def __call__(self, words):
        word_cache, sid_cache = _word_cache, _sid_cache
        for word in words:
            used = []
            for option in getattr(word, 'origins', []) + [word]:
                if option in used:
                    continue
                used.append(option)
                for sid in word_cache[word]:
                    for synonym in sid_cache[sid]:
                        yield TransformedWord(synonym, word)
            yield TransformedWord(word, word)
