# encoding: utf-8
"""
Methods used for looking up synonyms in WordNet (Norwegian)
"""

from __future__ import with_statement

from collections import defaultdict
import os

from clirch.utils.word import TransformedWord

DEFAULT_NOUNFILE=os.path.join(os.path.dirname(__file__), 'ordnett_nouns_big.csv')
DEFAULT_SENSEFILE=os.path.join(os.path.dirname(__file__), 'ordnett_relations.csv')

_bid_cache = dict()
_word_cache = defaultdict(list)

class NorNetSynonyms(object):

    def __init__(self, sensefile=None, bidfile=None):
        self.sensefile = sensefile or DEFAULT_SENSEFILE
        self.bidfile = bidfile or DEFAULT_NOUNFILE

        global _bid_cache, _word_cache
        if _word_cache:
            return

        bid_cache, word_cache = _bid_cache, _word_cache

        with open(self.bidfile) as bids:
            next(bids)
            for line in bids:
                bet_id, _, lemma = line.decode('latin1').split('\t')
                bid_cache[bet_id.strip()] = lemma.strip()

        with open(self.sensefile) as senses:
            next(senses)
            for line in senses:
                _, bid1, reltype, bid2 = line.decode('latin1').split('\t')
                if reltype.strip() == 'has_synonym':
                    word_cache[bid_cache[bid1.strip()]].append(bid_cache[bid2.strip()])
                    word_cache[bid_cache[bid2.strip()]].append(bid_cache[bid1.strip()])

    def __call__(self, words):
        word_cache = _word_cache
        for word in words:
            used = []
            for option in getattr(word, 'origins', []) + [word]:
                if option in used:
                    continue
                used.append(option)
                for synonym in word_cache[word]:
                    yield TransformedWord(synonym, word)
            yield TransformedWord(word, word)
