# encoding: utf-8

"""
A module for holding methods that do pruning of some sort.
"""

from collections import defaultdict
from operator import itemgetter

from nltk import lazyimport, util

reverse = lazyimport.LazyModule('reverse', locals())

def eliminate_duplicates(elements):
    yielded = set()
    for element in elements:
        if not element in yielded:
            yield element
            yielded.add(element)

def reduce_options_by_group(group_by=0, count=3):
    def wrapped_reduce_options_by_group(elements):
        groups = defaultdict(lambda: util.OrderedDict(default_factory=int))
        for element in elements:
            groups[element.origins[group_by]][element] += 1
        for group, elems in groups.iteritems():
            result = sorted(elems, key=lambda x: elems[x] * getattr(x, 'weight', 1.), reverse=True)[:count]
            weight = 1.0/sum(elems[elem] for elem in result)
            for elem in result:
                elem.weight = getattr(elem, 'weight', 1.0) * elems[elem] * weight
                yield elem
    return wrapped_reduce_options_by_group
