# encoding: utf-8

"""
Holds functionality for reverse pruning of words. Loosely based on the ideas
detailed in [Aljlayl et al, 2002] according to [Grossman & Frieder, 2004]
"""

import copy

from clirch.core import logger

class ReversePruner(object):

    def __init__(self, dictionary, level=0):
        self.lookup = dictionary
        self.level = level

    def __call__(self, words):
        lookup, level = self.lookup, self.level
        seen_origins = set()
        used_origins = set()

        for word in words:
            logger.debug('ReversePruner: origins: %s' % repr(word.origins))
            origin = word.origins[level]
            seen_origins.add(origin)
            reverse = set(lookup([word]))
            if origin in reverse:
                logger.debug('ReversePruner: Passed %r, %r in %s' % (word, origin, reverse))
                used_origins.add(origin)
                yield word
            else:
                logger.debug('ReversePruner: Dropped %r, %r not in %s' % (word, origin, reverse))

        bonus = seen_origins - used_origins
        logger.debug("Yielding the terms that didn't get translated: %s" % bonus)
        for origin in bonus:
            yield origin

    def set_languages(self, orig_lang, prune_lang):
        if isinstance(self.lookup, type):
            self.lookup = self.lookup(prune_lang, orig_lang)
        else:
            self.lookup = copy.copy(self.lookup)
            self.lookup.set_languages(prune_lang, orig_lang)
