# encoding: utf-8

import logging

import nltk.corpus
import nltk.stem
from nltk.tokenize import wordpunct_tokenize as word_tokenize

from word import TransformedWord
from mtag_parser import mtag
from treetag_parser import treetag
from freeling_parser import freeling

from clirch.core import logger

_filtrers = dict()
_stemmers = dict()

def stemmer_wrapper(stemmer_funtion):
    def stemmed_word_generator(words):
        for word in words:
            stem = stemmer_funtion(word)
            if not stem:
                yield TransformedWord(word, word)
            else:
                yield TransformedWord(stem, word)
    return stemmed_word_generator

_stemmers['English'] = stemmer_wrapper(nltk.corpus.wordnet.morphy)

def get_stopword_filtrer(language):
    "Returns an function that can remove stopwords for the given language."
    language = language.lower()
    if language in _filtrers:
        return _filtrers[language]

    stopword_set = set(unicode(word, 'utf-8') for word in nltk.corpus.stopwords.words(language))

    def stopword_filtrer(words):
        for word in words:
            if not word in stopword_set:
                yield word

    _filtrers[language] = stopword_filtrer
    return stopword_filtrer

def get_stemmer(language):
    "Returns a function that can stem words for the given language."
    if language in _stemmers:
        return _stemmers[language]

    stemmer = getattr(nltk.stem.snowball, '%sStemmer' % language)()

    _stemmers[language] = stemmer_wrapper(stemmer.stem)
    return _stemmers[language]

def force_unicode(string):
    if isinstance(string, unicode):
        return string

    try:
        return unicode(string, 'utf-8')

    except UnicodeDecodeError:
        return unicode(string, 'iso-8859-1')

def flatten_structure(elements, levels=100):
    if not hasattr(elements, '__iter__'):
        yield elements
    elif levels <= 0:
        for element in elements:
            yield element
    else:
        for element in elements:
            for subelement in flatten_structure(element, levels - 1):
                yield subelement

def get_regexp_tokenizer(regexp=r'(?<=")([^"]+)(?=")|(\w+)'):
    "Returns a callable object that tokenizes based on the input regex"

    tokenizer = nltk.tokenize.regexp.RegexpTokenizer(regexp)

    return tokenizer.tokenize
