# encoding: utf-8

import subprocess

from nltk.internals import find_binary

from clirch.core import logger
from clirch.utils.word import TransformedWord

ASTURIAN_CONFIG = '/usr/share/FreeLing/config/as.cfg'
CATALAN_CONFIG = '/usr/share/FreeLing/config/ca.cfg'
ENGLISH_CONFIG = '/usr/share/FreeLing/config/en.cfg'
GALICIAN_CONFIG = '/usr/share/FreeLing/config/gl.cfg'
ITALIAN_CONFIG = '/usr/share/FreeLing/config/it.cfg'
PORTUGUESE_CONFIG = '/usr/share/FreeLing/config/pt.cfg'
SPANISH_CONFIG = '/usr/share/FreeLing/config/es.cfg'
WELSCH_CONFIG = '/usr/share/FreeLing/config/cy.cfg'

LANGS = {
    'as': ASTURIAN_CONFIG,
    'ca': CATALAN_CONFIG,
    'en': ENGLISH_CONFIG,
    'gl': GALICIAN_CONFIG,
    'it': ITALIAN_CONFIG,
    'pt': PORTUGUESE_CONFIG,
    'es': SPANISH_CONFIG,
    'cy': WELSCH_CONFIG
    }

def freeling(lang='es'):

    def freeling_parse(in_string):
        if hasattr(in_string, '__iter__'):
            in_string = u' '.join(in_string)
        tagger = subprocess.Popen([
                find_binary('analyze', env_vars='FREELING', verbose=False),
                '--utf', '-f', LANGS.get(lang, lang)
            ],
            stdin=subprocess.PIPE, stderr=subprocess.PIPE,
            stdout=subprocess.PIPE)
        out, err = tagger.communicate(in_string.encode('utf-8'))
        if err:
            raise RuntimeError(err)

        for line in out.splitlines():
            line = line.strip().decode('utf-8')
            if not line:
                continue
            word, lemma, _, _ = line.split()
            if lemma[0].isalpha():
                yield TransformedWord(lemma, word)

    return freeling_parse

if __name__ == '__main__':
    import sys
    # python freeling_parser.py ¿No ibas a cogerla?
    print ' '.join(unicode(word) for word in freeling()(' '.join(sys.argv[1:]).decode('utf-8')))
