# encoding: utf-8

import subprocess

from nltk.internals import find_binary

from clirch.core import logger
from clirch.utils.word import TransformedWord

def mtag(in_string):
    if hasattr(in_string, '__iter__'):
        in_string = u' '.join(in_string)
    tagger = subprocess.Popen(find_binary('mtag', env_vars='MTAG_PATH',
        verbose=False), stdin=subprocess.PIPE, stderr=subprocess.PIPE,
        stdout=subprocess.PIPE)
    out, err = tagger.communicate(in_string.encode('utf-8'))
    if err:
        raise RuntimeError(err)

    variants = set()
    current_word = ''

    for line in out.splitlines():
        # print line.rstrip()
        line = line.strip()
        if line.startswith('"<'):
            if current_word:
                for variant in variants:
                    yield TransformedWord(variant, current_word)
            current_word = unicode(line[2:-2], 'utf-8').lower().strip(u'"«»,;\'')
            variants = set()
        elif line.startswith('"'):
            _, word, desc = line.split('"', 2)
            if not word.startswith('$') and not desc.endswith('symb'):
                word = unicode(word, 'utf-8').lower().strip(u'"«»,;\'')
                if word:
                    variants.add(word)

    for variant in variants:
        yield TransformedWord(variant, current_word)

if __name__ == '__main__':
    import sys
    # python mtag_parser.py dette er norske ord.
    print ' '.join(unicode(word) for word in mtag(' '.join(sys.argv[1:]).decode('utf-8')))
