# encoding: utf-8

import itertools
import re
import subprocess

from nltk.internals import find_binary

from clirch.core import logger
from clirch.utils.word import TransformedWord

def treetag(in_string, keep_origin=False):
    if hasattr(in_string, '__iter__'):
        in_string = u' '.join(in_string)
    tagger = subprocess.Popen(find_binary('tree-tagger-spanish-utf8',
        env_vars='TREETAG_PATH', verbose=False), stdin=subprocess.PIPE,
        stderr=subprocess.PIPE, stdout=subprocess.PIPE)
    out, _ = tagger.communicate(in_string.encode('utf-8'))

    current_phrase = None
    current_origin = []
    for line in out.splitlines():
        print line.rstrip()
        try:
            origin, pos, variants = unicode(line, 'utf-8').lower().split('\t', 2)
        except ValueError:
            continue

        results = set()
        first = True
        if current_phrase is not None:
            current_origin.append(origin)
        for variant in variants.split('|'):
            if pos == 'qt':
                if current_phrase is None:
                    current_phrase = []
                    current_origin = [origin]
                else:
                    for phrase in itertools.product(*current_phrase):
                        results.add(' '.join(phrase).replace('~', ' '))
                    current_phrase = None
            elif variant.isalnum() or '~' in variant:
                if current_phrase is None:
                    results.add(variant.replace('~', ' '))
                else:
                    if first:
                        current_phrase.append([])
                    current_phrase[-1].append(variant)
            else:
                # We do this in case it's an unknown named entity with a space
                # in it.
                for part in origin.split():
                    # But ¡we don't want any punctiation or weird stuff!
                    if part.isalnum():
                        if current_phrase is None:
                            results.add(part)
                        else:
                            if first:
                                current_phrase.append([])
                            current_phrase[-1].append(part)
            first = False
        if current_origin and current_phrase is None:
            origin = ' '.join(current_origin)
            current_origin = None
        if origin in results and not keep_origin:
            results.remove(origin)
        if results:
            for variant in results:
                yield TransformedWord(variant, origin)
        elif re.match(r'\w', origin, re.U):
            yield TransformedWord(origin, origin)

if __name__ == '__main__':
    import sys
    # python treetag_parser.py ¿No ibas a cogerla?
    print ' '.join(unicode(word) for word in treetag(' '.join(sys.argv[1:]).decode('utf-8')))
